      Program Local
C
C     Programer: Zweistein, Nov. 14, 2021
C     Local.for creates local systems for every element parallel to the maximum principal stress fom "Static.inp"
C     Theory: Principal stress angle with respect to cartesian system from Mohrs circle
C     Applicable for 2D plane stress problems
C
C     Input: Static.dat file from Calculix Job with Static.inp
C     Output: Elset.inp for every element: *Elset, Elset=Exxx, xxx = 1... Maximum Number of Elements
C     Output: Solid.inp for every element: *Solid Section, Material=Mat, Orientation=ORxxx, Elset=Exxx
C     Output: Orientation.out for every element: *Orientation, Name=ORxxx
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PI = 3.141592654	
C
      Open (Unit=11,File='Static.dat',      Status='old')  	! Input: dat file from Calculix Job with Static.inp
      Open (Unit=30,File='Elset.inp',       Status='Unknown')  	! Output *Elset
      Open (Unit=31,File='Solid.inp',       Status='Unknown')  	! Output *Solid Section
      Open (Unit=32,File='Orientation.inp', Status='Unknown')  	! Output *Orientation
C
      Read  (11, 999)			 			! Jump over 3 lines in file Static.dat
      Read  (11, 999)
      Read  (11, 999)
C
   10 Read  (11,*,END=20) NumEl, IntPt, Sxx, Syy, Szz, Sxy	! Read relevant 2D-stresses Sxx, Syy, Sxy
      Write (30,3000) NumEl, NumEl
C								! Principal stress angle from Mohrs circle
      Root  = 0.25*(Sxx-Syy)**2 + Sxy**2
C     Smax  = 0.5* (Sxx+Syy) + SQRT(Radik)			! Maximum principal stress, not used
      Smin  = 0.5* (Sxx+Syy) - SQRT(Root)			! Minimum principal stress
C
      SxSm = Sxx - Smin						! Difference Sxx-Smin
      If (ABS(SxSm) .LT. 1.D-10) SxSm = SIGN(1.D-10, SxSm)	! SxSm-Test on "zero" 
      Alfa = 180.*ATAN(Sxy/SxSm)/PI				! 1st principal stress angle (degree)
      Beta = ALfa+90.						! 2nd principal stress angle, not used
C
      Write (31,3001) NumEl, NumEl	! *Solid Section
      Write (32,3002) NumEl		! *Orientation
      Write (32,3003)     		! 1., 0., 0.,  1., 0., 0.  (cartesian system)
      Write (32,3004) Alfa		! Additinal rotation Alfa about z
      Go To 10
   20 Continue
C
  999 Format(5X)
 1000 Format (I7)
 3000 Format ('*Elset, Elset=E',I7/I7)
 3001 Format ('*Solid Section, Material=Mat,', 
     +        ' Orientation=OR',I7,', Elset=E',I7)
 3002 Format ('*Orientation, Name=OR',I7)
 3003 Format ('1., 0., 0.,  0., 1., 0.')
 3004 Format ('3,  ',F10.4)
      END
